/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_BLOCK_FILTER_H_
#define __GDAM_BLOCK_FILTER_H_

typedef struct _GdamBlockFilterClass GdamBlockFilterClass;
typedef struct _GdamBlockFilter GdamBlockFilter;

#include "gdamfilter.h"

GtkType gdam_block_filter_get_type();
#define GDAM_TYPE_BLOCK_FILTER   gdam_block_filter_get_type()
#define GDAM_BLOCK_FILTER(ptr) ((GdamBlockFilter*) gtk_type_check_object_cast ((GtkTypeObject*) (ptr), GDAM_TYPE_BLOCK_FILTER))
#define GDAM_BLOCK_FILTER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_BLOCK_FILTER, GdamBlockFilterClass))
#define GDAM_IS_BLOCK_FILTER(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_BLOCK_FILTER))
#define GDAM_IS_BLOCK_FILTER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_BLOCK_FILTER))

struct _GdamBlockFilterClass {
	GdamFilterClass		filter_class;
	int 		      (*process_block)(GdamBlockFilter*   filter,
	                                       GdamSample*        output,
					       const GdamSample*  input);
	void                  (*set_block_sizes)(GdamBlockFilter* block_sizes,
	                                       int                input_size,
					       int                output_size);
};

struct _GdamBlockFilter {
	GdamFilter		base;
	int			input_block_size;
	int			output_block_size;

	GdamSample* 		output_block_buffer;
	GdamSample*		input_block_buffer;
	GdamSample*		output_at;
	int			cur_input_length;
	int			cur_output_length;

	/* Samples missed via `miss_data' */
	int			missed;
};

void gdam_block_filter_set_sizes(GdamBlockFilter* filter,
                                 int              input_sizes,
                                 int              output_sizes);
#endif
