/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_BUFFERING_FILTER_H_
#define __GDAM_BUFFERING_FILTER_H_

typedef struct _GdamBufferingFilter GdamBufferingFilter;
typedef struct _GdamBufferingFilterClass GdamBufferingFilterClass;
typedef struct _GdamBufferingFilterElement GdamBufferingFilterElement;

#include "gdamfilter.h"

GtkType gdam_buffering_filter_get_type();
#define GDAM_TYPE_BUFFERING_FILTER   gdam_buffering_filter_get_type()
#define GDAM_BUFFERING_FILTER(ptr) ((GdamBufferingFilter*) gtk_type_check_object_cast ((GtkTypeObject*) (ptr), GDAM_TYPE_BUFFERING_FILTER))

struct _GdamBufferingFilterClass {
	GdamFilterClass		filter_class;
};

struct _GdamBufferingFilter {
	GdamFilter		filter;
	int			buffer_length;

	/* Invariant: 0 <= buffer_inbound_pos < buffer_length */
	int			buffer_inbound_pos;

	int			buffer_current_amount;
	GdamSample*		buffer;
};

void gdam_buffering_filter_buffer(GdamBufferingFilter* buffering_filter,
				  const GdamSample* in, int n);

int gdam_buffering_filter_buffer_from_source(GdamBufferingFilter* filter,
				              GdamSource* src, int max);

/* The number returned is the number of samples discarded
 * from the filter's buffer to make room for the n bytes of buf. */
int  gdam_buffering_filter_append(GdamBufferingFilter* filter,
				  const GdamSample*    buf,
				  int                  n);

void gdam_buffering_filter_ensure_space(GdamBufferingFilter* filter,
		                        int       	     amount);

int gdam_buffering_filter_peek_from_buffer(GdamBufferingFilter* filter,
					   GdamSample* dest, int n);

static inline void gdam_buffering_filter_append_sample
						(GdamBufferingFilter *filter,
						 GdamSample           sample)
{
	if (filter->buffer_length == 0)
		return;
	++(filter->buffer_inbound_pos);
	if (filter->buffer_inbound_pos == filter->buffer_length)
		filter->buffer_inbound_pos = 0;
	filter->buffer[filter->buffer_inbound_pos] = sample;
	if (filter->buffer_current_amount < filter->buffer_length)
		filter->buffer_current_amount++;
}
	

static inline GdamSample gdam_buffering_filter_sample_at
						(GdamBufferingFilter *filter,
						 int                  delay)
{
	int pos = filter->buffer_inbound_pos - delay;
	if (pos < 0)
		pos += filter->buffer_length;
	return filter->buffer[pos];
}

static inline GdamSample gdam_buffering_filter_sample_from_start
						(GdamBufferingFilter *filter,
						 int                  offset)
{
	int pos = filter->buffer_inbound_pos
	        - filter->buffer_current_amount
		+ offset;
	if (pos < 0)
		pos += filter->buffer_length;
	return filter->buffer[pos];
}

#endif
