/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamsynthdcblock.h"
#include <gdam/gdamplugin.h>

static int gdam_synth_dc_block_synthesize(GdamSynthSource* source,
                                          gdouble*         buf,
					  int              num)
{
	GdamSynthDCBlock* dc_block = GDAM_SYNTH_DC_BLOCK(source);
	GdamSynthSource* filtered = ((GdamSynthFilter*)source)->filtered;
	int num_read = gdam_synth_source_synthesize(filtered, buf, num);
	gdouble last_output = dc_block->last_output;
	gdouble last_input = dc_block->last_input;
	num = num_read;
	while (num_read-- > 0) {
		gdouble cur = *buf;
		last_output = cur - last_input + 0.99 * last_output;
		last_input = cur;
		*buf++ = last_output;
	}
	dc_block->last_output = last_output;
	dc_block->last_input = last_input;

	return num;
}

static void gdam_synth_dc_block_init(GdamSynthDCBlock* synth_dc_block)
{
}
static void gdam_synth_dc_block_class_init(GdamSynthSourceClass* klass)
{
	klass->synthesize = gdam_synth_dc_block_synthesize;
}

/* The _d_c_ is *needed* to automatically find this function in this
 * plugin module, unaesthetic as it may be... */
GtkType gdam_synth_d_c_block_get_type()
{
	static GtkType synth_dc_block_type = 0;
	if (!synth_dc_block_type) {
		GtkType parent_type;
		static const GtkTypeInfo synth_dc_block_info = {
			"GdamSynthDCBlock",
			sizeof(GdamSynthDCBlock),
			sizeof(GdamSynthDCBlockClass),
			(GtkClassInitFunc) gdam_synth_dc_block_class_init,
			(GtkObjectInitFunc) gdam_synth_dc_block_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		parent_type = gdam_plugin_get_type_by_name("server",
		                                           "GdamSynthFilter");
		synth_dc_block_type = gtk_type_unique(parent_type,
		                                 &synth_dc_block_info);
	}
	return synth_dc_block_type;
}
