/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_SYNTH_DC_BLOCK_H_
#define __GDAM_SYNTH_DC_BLOCK_H_

/* DC Blocking filter.
 *
 * based after that in Perry Cook's STK library.
 * As he says:

		 This guy is very helpful in, uh,
		 blocking DC.  Needed because a simple
		 low-pass reflection filter allows DC
		 to build up inside recursive
		 structures.
 
 */

typedef struct _GdamSynthDCBlock GdamSynthDCBlock;
typedef struct _GdamSynthDCBlockClass GdamSynthDCBlockClass;

#include "gdamsynthfilter.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdam_synth_dc_block_get_type();
#define GDAM_TYPE_SYNTH_DC_BLOCK	      (gdam_synth_dc_block_get_type ())
#define GDAM_SYNTH_DC_BLOCK(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_SYNTH_DC_BLOCK, GdamSynthDCBlock))
#define GDAM_SYNTH_DC_BLOCK_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_SYNTH_DC_BLOCK, GdamSynthDCBlockClass))
#define GDAM_IS_SYNTH_DC_BLOCK(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_SYNTH_DC_BLOCK))
#define GDAM_IS_SYNTH_DC_BLOCK_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_SYNTH_DC_BLOCK))

struct _GdamSynthDCBlockClass {
	GdamSynthFilterClass		synth_dc_block_class;
};
struct _GdamSynthDCBlock {
	GdamSynthFilter			synth_dc_block;

	gdouble				last_input;
	gdouble				last_output;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
